package com.softslate.commerce.administrator.order;

import java.util.Arrays;
import java.util.Collection;
import java.util.Map;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.apache.commons.beanutils.PropertyUtils;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.apache.struts.action.ActionForm;
import org.apache.struts.action.ActionForward;
import org.apache.struts.action.ActionMapping;

import com.softslate.commerce.businessobjects.core.BusinessObjectFactory;
import com.softslate.commerce.businessobjects.order.Order;
import com.softslate.commerce.businessobjects.order.OrderDiscountProcessor;
import com.softslate.commerce.businessobjects.order.OrderProcessor;
import com.softslate.commerce.customer.core.BaseAction;

/**
 * Struts action class that corresponds to the main administrator view panel for order discounts. Used for requests to
 * "/administrator/OrderDiscount.do".
 * <p>
 * The <code>execute</code> method of this class populates a <code>Collection</code> named <code>itemCollection</code>
 * in the request scope containing the items to be displayed. Next, it sets a request scope variable named
 * <code>editPermission</code> that determines if the current user has permission to edit the items. Finally, it creates
 * a <code>Collection</code> named <code>breadcrumbs</code> containing the links and labels of each breadcrumb for
 * display.
 * 
 * @author David Tobey
 * 
 */
public class OrderDiscountAction extends BaseAction {

	static Log log = LogFactory.getLog(OrderDiscountAction.class);

	public ActionForward execute(ActionMapping mapping, ActionForm form,
			HttpServletRequest request, HttpServletResponse response)
			throws Exception {
		if (log.isDebugEnabled())
			log.debug("Starting action.");
		
		OrderDiscountForm baseForm = (OrderDiscountForm) form;
		baseForm.setProperties(request);
		request.setAttribute("mainForm", baseForm);

		// Get the collection of items to display
		BusinessObjectFactory bof = (BusinessObjectFactory) request
				.getAttribute("businessObjectFactory");
		OrderDiscountProcessor processor =  bof
				.createObject(OrderDiscountProcessor.class);
		Map orderDiscountsAndCount = processor
				.loadOrderDiscountsAndCount(PropertyUtils.describe(baseForm));
		request.setAttribute("orderDiscounts", orderDiscountsAndCount
				.get("items"));
		request.setAttribute("itemCollection", orderDiscountsAndCount
				.get("items"));
		request.setAttribute("itemCount", orderDiscountsAndCount
				.get("itemCount"));
		request.setAttribute("assignedItems", orderDiscountsAndCount
				.get("assignedItems"));

		// Load the parent if an id is present
		Order order = null;
		if (baseForm.getOrderID() != null && baseForm.getOrderID().length() > 0) {
			OrderProcessor parentProcessor =  bof
					.createObject(OrderProcessor.class);
			order = parentProcessor.loadOrderFromID(PropertyUtils
					.describe(baseForm));
			request.setAttribute("order", order);
		}

		// Enforce role permissions
		if (!request.isUserInRole("superuser")
				&& !request.isUserInRole("administrator")
				&& !request.isUserInRole("orderDiscountEdit")) {
			request.setAttribute("editPermission", "false");
			baseForm.setEditMode("false");
		} else {
			request.setAttribute("editPermission", "true");
		}

		// Create breadcrumbing collection
		Collection bcs = Arrays.asList(new String[] { "Home", "Orders",
				"Order Details", "Order Discounts" });
		request.setAttribute("breadcrumbs", bcs);

		// Forward according to the outputMode of the request
		if (baseForm.getOutputMode().equals("print")) {
			request.setAttribute("outputMode", "print");
			baseForm.setOutputMode("screen");
			return mapping.findForward("successSimpleLayout");
		} else if (baseForm.getOutputMode().equals("excel")) {
			request.setAttribute("outputMode", "excel");
			baseForm.setOutputMode("screen");
			response.setHeader("Content-Type", "application/vnd.ms-excel");
			response.setHeader("Content-Disposition",
					"attachment; filename=orderDiscounts.xls");
			return mapping.findForward("successSimpleLayout");
		}
		return mapping.findForward("success");
	}
}
