package com.softslate.commerce.administrator.product;

import java.util.Arrays;
import java.util.Collection;
import java.util.HashMap;
import java.util.Map;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.apache.commons.beanutils.PropertyUtils;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.apache.struts.action.ActionForm;
import org.apache.struts.action.ActionForward;
import org.apache.struts.action.ActionMapping;

import com.softslate.commerce.businessobjects.core.BusinessObjectFactory;
import com.softslate.commerce.businessobjects.product.CategoryProcessor;
import com.softslate.commerce.businessobjects.product.Product;
import com.softslate.commerce.businessobjects.product.ProductProcessor;
import com.softslate.commerce.customer.core.BaseAction;

/**
 * Struts action class that corresponds to the main administrator view panel for
 * categories. Used for requests to "/administrator/Category.do".
 * <p>
 * The <code>execute</code> method of this class populates a
 * <code>Collection</code> named <code>itemCollection</code> in the request
 * scope containing the items to be displayed. Next, it sets a request scope
 * variable named <code>editPermission</code> that determines if the current
 * user has permission to edit the items. Finally, it creates a
 * <code>Collection</code> named <code>breadcrumbs</code> containing the
 * links and labels of each breadcrumb for display.
 * 
 * @author David Tobey
 * 
 */
public class CategoryAction extends BaseAction {

	static Log log = LogFactory.getLog(CategoryAction.class);

	public ActionForward execute(ActionMapping mapping, ActionForm form,
			HttpServletRequest request, HttpServletResponse response)
			throws Exception {
		if (log.isDebugEnabled())
			log.debug("Starting action.");

		// Handle the main form for the control screen
		CategoryForm baseForm = (CategoryForm) form;
		baseForm.setProperties(request);
		request.setAttribute("mainForm", baseForm);

		// Only show link to "Details" for Content Pages
		if (baseForm.getWorkWithCategories() != null
				&& baseForm.getWorkWithCategories().equals("Content Pages")) {
			baseForm.setEditFormAction("/CategoryEditForm.do");
			baseForm.setChildEditAction(null);
			baseForm.setChildName(null);
		}

		// Handle the quick-add form
		CategoryAddEditForm addForm = (CategoryAddEditForm) request
				.getAttribute("categoryAddEditForm");
		if (addForm == null
				|| (request.getAttribute("reset") != null && request
						.getAttribute("reset").equals("true"))) {
			addForm = new CategoryAddEditForm();
		}
		addForm.initializeProperties(mapping, request);
		request.setAttribute("categoryAddEditForm", addForm);

		// Get the collection of items to display
		BusinessObjectFactory bof = (BusinessObjectFactory) request
				.getAttribute("businessObjectFactory");
		CategoryProcessor processor =  bof
				.createObject(CategoryProcessor.class);
		Map categoriesAndCount = processor.loadCategoriesAndCount(PropertyUtils
				.describe(baseForm));
		request.setAttribute("categories", categoriesAndCount.get("items"));
		request.setAttribute("itemCollection", categoriesAndCount.get("items"));
		request.setAttribute("itemCount", categoriesAndCount.get("itemCount"));
		request.setAttribute("assignedItems", categoriesAndCount
				.get("assignedItems"));

        // Bug where the parentCategory field gets renamed to parent
        for (int j = 0; j < baseForm.getDisplayFields().length; j++) {
            String displayField = baseForm.getDisplayFields()[j];
            if (displayField.equals("parent")) {
            	displayField = "parentCategory";
            	baseForm.getDisplayFields()[j] = "parentCategory";
            }
        }

		// Load all categories for the parent category drop down.
		if (baseForm.getEditMode() != null
				&& baseForm.getEditMode().equals("true")
				&& Arrays.asList(baseForm.getDisplayFields()).contains(
						"parentCategory")) {
			Collection categories = processor.loadAllCategories();
			request.setAttribute("categories", categories);
		}

		// Load the parent if an id is present
		Product product = null;
		if (baseForm.getProductID() != null
				&& baseForm.getProductID().length() > 0) {
			ProductProcessor productProcessor =  bof
					.createObject(ProductProcessor.class);
			Map parameters = new HashMap();
			parameters.put("productID", baseForm.getProductID());
			product = productProcessor.loadProductFromID(parameters);
			request.setAttribute("product", product);
		}

		// Enforce role permissions
		if (!request.isUserInRole("superuser")
				&& !request.isUserInRole("administrator")
				&& !request.isUserInRole("categoryEdit")) {
			request.setAttribute("editPermission", "false");
			baseForm.setEditMode("false");
		} else {
			request.setAttribute("editPermission", "true");
		}

		if (baseForm.getProductID() != null
				&& baseForm.getProductID().length() > 0) {

			// Create breadcrumbing collection
			Collection bcs = Arrays.asList(new String[] { "Home", "Products",
					"Product Details", "Categories" });
			if (baseForm.getWorkWithCategories() != null
					&& baseForm.getWorkWithCategories().equals("Content Pages")) {
				bcs = Arrays.asList(new String[] { "Home", "Products",
						"Product Details", "Content Pages" });
			}
			request.setAttribute("breadcrumbs", bcs);

		} else {

			// Create breadcrumbing collection
			Collection bcs = Arrays
					.asList(new String[] { "Home", "Categories" });
			if (baseForm.getWorkWithCategories() != null
					&& baseForm.getWorkWithCategories().equals("Content Pages")) {
				bcs = Arrays.asList(new String[] { "Home", "Content Pages" });
			}
			request.setAttribute("breadcrumbs", bcs);

		}

		// Forward according to the outputMode of the request
		if (baseForm.getOutputMode().equals("print")) {
			request.setAttribute("outputMode", "print");
			baseForm.setOutputMode("screen");
			return mapping.findForward("successSimpleLayout");
		} else if (baseForm.getOutputMode().equals("excel")) {
			request.setAttribute("outputMode", "excel");
			baseForm.setOutputMode("screen");
			response.setHeader("Content-Type", "application/vnd.ms-excel");
			response.setHeader("Content-Disposition",
					"attachment; filename=categorys.xls");
			return mapping.findForward("successSimpleLayout");
		}
		return mapping.findForward("success");
	}
}
