package com.softslate.commerce.administrator.product;

import java.util.Map;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.apache.commons.beanutils.PropertyUtils;
import org.apache.commons.lang3.StringEscapeUtils;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.apache.struts.action.ActionForm;
import org.apache.struts.action.ActionForward;
import org.apache.struts.action.ActionMapping;
import org.apache.struts.action.ActionMessage;
import org.apache.struts.action.ActionMessages;

import com.softslate.commerce.businessobjects.product.ProductSetting;
import com.softslate.commerce.businessobjects.product.ProductSettingProcessor;
import com.softslate.commerce.customer.core.BaseAction;

/**
 * Struts action class that corresponds to requests to "/administrator/ProductSettingAdd.do". Processes a request to add
 * a record to the system.
 * 
 * @author David Tobey
 */
public class ProductSettingAddAction extends BaseAction {

	static Log log = LogFactory.getLog(ProductSettingAddAction.class);

	public ActionForward execute(ActionMapping mapping, ActionForm form, HttpServletRequest request,
			HttpServletResponse response) throws Exception {
		log.info("Starting action.");

		// Add the record
		ProductSettingAddEditForm baseForm = (ProductSettingAddEditForm) form;
		ProductSettingProcessor processor = baseForm.getBusinessObjectFactory()
				.createObject(ProductSettingProcessor.class);
		Map results = processor.addProductSetting(PropertyUtils.describe(form));

		// Successful insert
		if (results == null || results.get("resultCode") == null || results.get("resultCode").equals("0")) {

			// Set the success messages
			baseForm.getMessages().add(ActionMessages.GLOBAL_MESSAGE, new ActionMessage("messages.recordAdded"));
			if (results != null && results.get("newRecord") != null) {
				ProductSetting newRecord = (ProductSetting) results.get("newRecord");

				// entity.jsp wants the new record in the request scope.
				request.setAttribute("productSetting", newRecord);
			}

			// If successful, signal the form to be reinstantiated.
			request.setAttribute("reset", "true");

			if (log.isInfoEnabled() && !baseForm.getErrors().isEmpty()) {
				log.info("Validation error(s): " + baseForm.getErrors().toString());
			}
			saveMessages(baseForm.getRequest(), baseForm.getMessages());
			saveErrors(baseForm.getRequest(), (ActionMessages) baseForm.getErrors());
			return mapping.findForward("success");
		}

		// Set the error messages
		if (results != null && results.get("existingRecord") != null) {
			ProductSetting existingRecord = (ProductSetting) results.get("existingRecord");
			String duplicateField = (String) results.get("duplicateField");
			String existingCode = StringEscapeUtils.escapeXml11(existingRecord.getCode());
			request.setAttribute("okNotToEscapeMessages", Boolean.TRUE);
			baseForm.getErrors().add(ActionMessages.GLOBAL_MESSAGE,
					new ActionMessage("errors.resultCode" + results.get("resultCode"), duplicateField, existingCode));
		} else if (results != null && results.get("resultCode") != null
				&& results.get("resultCode").equals("NoParentFound")) {
			baseForm.getErrors().add(ActionMessages.GLOBAL_MESSAGE,
					new ActionMessage("errors.resultCode" + results.get("resultCode")));

		}
		if (log.isInfoEnabled() && !baseForm.getErrors().isEmpty()) {
			log.info("Validation error(s): " + baseForm.getErrors().toString());
		}
		saveMessages(baseForm.getRequest(), baseForm.getMessages());
		saveErrors(baseForm.getRequest(), (ActionMessages) baseForm.getErrors());
		return mapping.findForward("success");
	}
}
