package com.softslate.commerce.administrator.product;

import java.util.Map;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.apache.commons.lang3.StringEscapeUtils;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.apache.struts.action.ActionForm;
import org.apache.struts.action.ActionForward;
import org.apache.struts.action.ActionMapping;
import org.apache.struts.action.ActionMessage;
import org.apache.struts.action.ActionMessages;

import com.softslate.commerce.businessobjects.product.ProductSetting;
import com.softslate.commerce.businessobjects.product.ProductSettingProcessor;
import com.softslate.commerce.customer.core.BaseAction;

/**
 * Struts action class that corresponds to requests to "/administrator/ProductSettingEdit.do". Processes a request to
 * edit a record within the system.
 * 
 * @author David Tobey
 * 
 */
public class ProductSettingEditAction extends BaseAction {

	static Log log = LogFactory.getLog(ProductSettingEditAction.class);

	public ActionForward execute(ActionMapping mapping, ActionForm form, HttpServletRequest request,
			HttpServletResponse response) throws Exception {
		if (log.isInfoEnabled())
			log.info("Starting ProductSetting Edit Action.");
		ProductSettingAddEditForm baseForm = (ProductSettingAddEditForm) form;
		baseForm.initializeProperties(mapping, request);

		// Edit the record
		ProductSettingProcessor processor = (ProductSettingProcessor) baseForm.getBusinessObjectFactory()
				.createObject("productSettingProcessorImplementer");
		Map results = processor.editProductSetting(baseForm.mapOfIncomingParams());

		// Successful update
		if (results == null || results.get("resultCode") == null || results.get("resultCode").equals("0")) {

			// entity.jsp wants the updated record in the request scope
			request.setAttribute("productSetting", results.get("updatedRecord"));

			// Set the success messages
			baseForm.getMessages().add(ActionMessages.GLOBAL_MESSAGE, new ActionMessage("messages.recordSaved"));
			saveMessages(baseForm.getRequest(), baseForm.getMessages());
			if (log.isInfoEnabled() && !baseForm.getErrors().isEmpty()) {
				log.info("Validation error(s): " + baseForm.getErrors().toString());
			}
			saveErrors(baseForm.getRequest(), (ActionMessages) baseForm.getErrors());
			return mapping.findForward("success");
		}

		// Set the error messages
		if (results != null && results.get("existingRecord") != null) {
			ProductSetting existingRecord = (ProductSetting) results.get("existingRecord");
			String duplicateField = (String) results.get("duplicateField");
			String existingCode = StringEscapeUtils.escapeXml11(existingRecord.getCode());
			request.setAttribute("okNotToEscapeMessages", Boolean.TRUE);
			baseForm.getErrors().add(ActionMessages.GLOBAL_MESSAGE,
					new ActionMessage("errors.resultCode" + results.get("resultCode"), duplicateField, existingCode));
		}
		if (log.isInfoEnabled() && !baseForm.getErrors().isEmpty()) {
			log.info("Validation error(s): " + baseForm.getErrors().toString());
		}
		saveErrors(baseForm.getRequest(), (ActionMessages) baseForm.getErrors());
		return mapping.findForward("success");
	}
}
