package com.softslate.commerce.administrator.product;

import java.util.Map;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.apache.commons.beanutils.PropertyUtils;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.apache.struts.action.ActionErrors;
import org.apache.struts.action.ActionForm;
import org.apache.struts.action.ActionForward;
import org.apache.struts.action.ActionMapping;
import org.apache.struts.action.ActionMessage;
import org.apache.struts.action.ActionMessages;

import com.softslate.commerce.businessobjects.core.BusinessObjectFactory;
import com.softslate.commerce.businessobjects.product.SKUProcessor;
import com.softslate.commerce.customer.core.BaseAction;
import com.softslate.commerce.customer.product.ProductInitializer;

/**
 * Struts action class that corresponds to requests to "/administrator/SKUAssign.do". Processes a request to assign a
 * SKU to a parent.
 * 
 * @author David Tobey
 */
public class SKUAssignAction extends BaseAction {

	static Log log = LogFactory.getLog(SKUAssignAction.class);

	public ActionForward execute(ActionMapping mapping, ActionForm form, HttpServletRequest request,
			HttpServletResponse response) throws Exception {
		if (log.isDebugEnabled())
			log.debug("Starting action.");
		SKUForm baseForm = (SKUForm) form;

		// Validate here because we only need to check a couple of things and SKUForm.validate is used by
		// SKUEditDeleteAction
		ActionErrors errors = new ActionErrors();
		if ((baseForm.getSkuNumber() == null || baseForm.getSkuNumber()[0].equals(""))
				&& (baseForm.getSkuID() == null || baseForm.getSkuID()[0].equals(""))) {
			errors.add(ActionMessages.GLOBAL_MESSAGE, new ActionMessage("errors.required", "SKU Number or SKU ID"));
		}
		if ((baseForm.getShippingRuleID() == null || baseForm.getShippingRuleID().equals(""))
				&& (baseForm.getDiscountID() == null || baseForm.getDiscountID().equals(""))) {
			errors.add(ActionMessages.GLOBAL_MESSAGE,
					new ActionMessage("errors.required", "shippingRuleID ID or discountID"));
		}
		if (!errors.isEmpty()) {
			saveErrors(request, errors);
			return mapping.findForward("success");
		}

		// Prepare parameters for the edit/delete processing, and process
		baseForm.setProperties(request);
		BusinessObjectFactory bof = (BusinessObjectFactory) request.getAttribute("businessObjectFactory");
		SKUProcessor processor = bof.createObject(SKUProcessor.class);
		Map parameters = PropertyUtils.describe(form);
		Map results = processor.editSKUAssignment(parameters);
		if (results != null && results.get("resultCode") != null && results.get("resultCode").equals("NotFound")) {
			errors.add(ActionMessages.GLOBAL_MESSAGE,
					new ActionMessage("errors.detail", "Record not found"));
			saveErrors(request, errors);
		} else {

			// Reload the store discounts in the application scope since the sku
			// associations may have changed
			ProductInitializer pi = new ProductInitializer();
			pi.setServlet(servlet);
			pi.initializeStoreDiscounts(bof);

			// Set the success message
			ActionMessages messages = new ActionMessages();
			messages.add(ActionMessages.GLOBAL_MESSAGE, new ActionMessage("messages.recordsSaved"));
			saveMessages(request, messages);
		}

		return mapping.findForward("success");
	}
}
