package com.softslate.commerce.administrator.product;

import java.util.Collection;
import java.util.Map;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.apache.commons.lang3.StringEscapeUtils;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.apache.struts.action.ActionForm;
import org.apache.struts.action.ActionForward;
import org.apache.struts.action.ActionMapping;
import org.apache.struts.action.ActionMessage;
import org.apache.struts.action.ActionMessages;

import com.softslate.commerce.businessobjects.core.ApplicationMessage;
import com.softslate.commerce.businessobjects.product.SKU;
import com.softslate.commerce.businessobjects.product.SKUProcessor;
import com.softslate.commerce.customer.core.ActionUtils;
import com.softslate.commerce.customer.core.BaseAction;

/**
 * Struts action class that corresponds to requests to
 * "/administrator/SKUEdit.do". Processes a request to edit a record within the
 * system.
 * 
 * @author David Tobey
 */
public class SKUEditAction extends BaseAction {

    static Log log = LogFactory.getLog(SKUEditAction.class);

    public ActionForward execute(ActionMapping mapping, ActionForm form,
            HttpServletRequest request, HttpServletResponse response)
            throws Exception {
        if (log.isDebugEnabled())
            log.debug("Starting action.");

		SKUAddEditForm baseForm = (SKUAddEditForm) form;

		// Handle generating the number from the name if it is empty
		ActionUtils actionUtils = baseForm.getBusinessObjectFactory().createUtilsObject(ActionUtils.class);
		if (baseForm.getSKUID() == null
				&& ((baseForm.getSkuNumber() == null || baseForm.getSkuNumber().equals("")) && (baseForm.getSkuName() != null && !baseForm
						.getSkuName().equals("")))) {
			baseForm.setSkuNumber(actionUtils.generateCodeFromName(baseForm.getSkuName()));
		}

        // Edit the record
		SKUProcessor processor = (SKUProcessor) baseForm.getBusinessObjectFactory().createObject(
				"skuProcessorImplementer");

		// Preserve parameters that were set in baseForm.validate.
		Map parameters = baseForm.mapOfIncomingParams();
		if (baseForm.getAttributeID1() != null) {
			parameters.put("attributeID1", baseForm.getAttributeID1());
		}
		if (baseForm.getAttributeID2() != null) {
			parameters.put("attributeID2", baseForm.getAttributeID2());
		}
		if (baseForm.getAttributeID3() != null) {
			parameters.put("attributeID3", baseForm.getAttributeID3());
		}
		if (baseForm.getAttributeID4() != null) {
			parameters.put("attributeID4", baseForm.getAttributeID4());
		}
		if (baseForm.getAttributeID5() != null) {
			parameters.put("attributeID5", baseForm.getAttributeID5());
		}
		if (baseForm.getOptionID1() != null) {
			parameters.put("optionID1", baseForm.getOptionID1());
		}
		if (baseForm.getOptionID2() != null) {
			parameters.put("optionID2", baseForm.getOptionID2());
		}
		if (baseForm.getOptionID3() != null) {
			parameters.put("optionID3", baseForm.getOptionID3());
		}
		if (baseForm.getOptionID4() != null) {
			parameters.put("optionID4", baseForm.getOptionID4());
		}
		if (baseForm.getOptionID5() != null) {
			parameters.put("optionID5", baseForm.getOptionID5());
		}
		Map results = processor.editSKU(parameters);

		// Successful update
		if (results == null || results.get("resultCode") == null
				|| results.get("resultCode").equals("0")) {

			// entity.jsp wants the updated record in the request scope
			request.setAttribute("sku", results.get("updatedRecord"));
			baseForm.getMessages().add(ActionMessages.GLOBAL_MESSAGE, new ActionMessage("messages.recordSaved"));
			actionUtils.processApplicationMessages(baseForm.getErrors(), baseForm.getMessages(),
					(Collection<ApplicationMessage>) results.get("applicationMessages"));
			saveErrors(baseForm.getRequest(), baseForm.getErrors());
			saveMessages(baseForm.getRequest(), baseForm.getMessages());
			return mapping.findForward("success");
		}

		// Set the error messages
		if (results != null && results.get("existingRecord") != null) {
			SKU existingRecord = (SKU) results.get("existingRecord");
			String duplicateField = (String) results.get("duplicateField");
			String existingCode = existingRecord.getSkuNumber();
			if (duplicateField.equals("productID")) {
				existingCode = existingRecord.getProductID().toString();
			}
			existingCode = StringEscapeUtils.escapeXml11(existingCode);
			request.setAttribute("okNotToEscapeMessages", Boolean.TRUE);
			baseForm.getErrors().add(
					ActionMessages.GLOBAL_MESSAGE,
					new ActionMessage("errors.resultCode"
							+ results.get("resultCode"), duplicateField,
							existingCode));
			baseForm.getErrors().add(
					ActionMessages.GLOBAL_MESSAGE,
					new ActionMessage("errors.editExistingRecord",
							"SKUEditForm.do?skuID="
									+ existingRecord.getSkuID(),
							duplicateField, existingCode));
			saveErrors(baseForm.getRequest(), (ActionMessages) baseForm
					.getErrors());
		} else if (results != null && results.get("resultCode") != null
				&& results.get("resultCode").equals("NotFound")) {
			baseForm.getErrors().add(ActionMessages.GLOBAL_MESSAGE,
					new ActionMessage("errors.detail", "Record not found"));
			saveErrors(baseForm.getRequest(), (ActionMessages) baseForm.getErrors());
		}

		return mapping.findForward("success");
    }
}
