package com.softslate.commerce.businessobjects.events;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import com.google.common.eventbus.Subscribe;
import com.softslate.commerce.businessobjects.core.BaseBusinessProcessor;
import com.softslate.commerce.businessobjects.core.BusinessObjectFactory;

/* 
 * This class is a singleton maintained by EventBus. For thread safety, it simply creates a new instance of
 * the Trigger and hands off the processing to it.
 */
public class QboOrderSubscriberImpl extends BaseBusinessProcessor
		implements QboOrderSubscriber {

	static Log log = LogFactory.getLog(QboOrderSubscriberImpl.class);

	@Subscribe
	public void processNewOrder(NewOrderEvent noe) throws Exception {
		log.info("Subscriber for QBO new order.");
		BusinessObjectFactory bof = noe.getCartProcessor().getBusinessObjectFactory();
		QboOrderTrigger trigger = bof.createObject(QboOrderTrigger.class);
		trigger.processNewOrder(noe);
	}

	@Subscribe
	public void processOrderShipped(ChangedOrderEvent coe) throws Exception {
		log.info("Subscriber for QBO changed order.");
		BusinessObjectFactory bof = coe.getBusinessProcessor().getBusinessObjectFactory();
		QboOrderTrigger trigger = bof.createObject(QboOrderTrigger.class);
		trigger.processChangedOrder(coe);
	}
}
