package com.softslate.commerce.businessobjects.order;

import org.hibernate.envers.AuditTable;
import org.hibernate.envers.Audited;

import com.softslate.commerce.businessobjects.core.BusinessObject;
import com.softslate.commerce.businessobjects.product.Discount;

/**
 * Interface representing an order discount within the system. An order discount
 * can be thought of as a
 * {@link com.softslate.commerce.businessobjects.product.Discount} that has been
 * applied to a user's order, either on the level of the order as whole or on
 * the level of a sinlge order item.
 * <p>
 * When the application needs to create an instance that implements
 * <code>OrderDiscount</code>,
 * {@link com.softslate.commerce.businessobjects.core.BusinessObjectFactory}
 * finds the name of the Java class to instantiate from the
 * "orderDiscountImplementer" setting in the
 * <code>appComponents.properties</code> file.
 * <p>
 * The default "orderDiscountImplementer" is
 * {@link com.softslate.commerce.businessobjects.order.OrderDiscountBean}.
 * 
 * @author David Tobey
 * 
 * @see com.softslate.commerce.businessobjects.product.Discount
 * 
 */
@Audited
@AuditTable("AudOrderDiscount")
public interface OrderDiscount extends BusinessObject {
	abstract Discount getDiscount();

	abstract void setDiscount(Discount discount);

	abstract Order getOrder();

	abstract void setOrder(Order order);

	abstract OrderItem getOrderItem();

	abstract void setOrderItem(OrderItem orderItem);

	abstract String getCode();

	abstract void setCode(String code);

	abstract String getCouponCode();

	abstract void setCouponCode(String couponCode);

	abstract String getExtra1();

	abstract void setExtra1(String extra1);

	abstract String getExtra2();

	abstract void setExtra2(String extra2);

	abstract String getExtra3();

	abstract void setExtra3(String extra3);

	abstract String getItemsAppliesType();

	abstract void setItemsAppliesType(String itemsAppliesType);

	abstract String getItemsAppliesValue();

	abstract void setItemsAppliesValue(String itemsAppliesValue);

	abstract String getUsersAppliesType();

	abstract void setUsersAppliesType(String usersAppliesType);

	abstract String getUsersAppliesValue();

	abstract void setUsersAppliesValue(String usersAppliesValue);

	abstract String getCartItemsType();

	abstract void setCartItemsType(String cartItemsType);

	abstract String getCartItemsValue();

	abstract void setCartItemsValue(String cartItemsValue);

	abstract String getName();

	abstract void setName(String name);

	abstract Double getReward();

	abstract void setReward(Double reward);

	abstract String getRewardType();

	abstract void setRewardType(String rewardType);

	abstract Double getCeiling();

	abstract void setCeiling(Double ceiling);

	abstract Integer getDiscountID();

	abstract void setDiscountID(Integer discountID);

	abstract Double getFloor();

	abstract void setFloor(Double floor);

	abstract int getOrderDiscountID();

	abstract void setOrderDiscountID(int orderDiscountID);

	abstract int getOrderID();

	abstract void setOrderID(int orderID);

	abstract Integer getOrderItemID();

	abstract void setOrderItemID(Integer orderItemID);

	abstract Double getTotal();

	abstract void setTotal(Double total);

	abstract int getDiscountOrder();

	abstract void setDiscountOrder(int discountOrder);

}
