package com.softslate.commerce.businessobjects.order;

import java.util.Collection;
import java.util.Locale;

import org.hibernate.envers.AuditTable;
import org.hibernate.envers.Audited;
import org.hibernate.envers.NotAudited;

import com.softslate.commerce.businessobjects.core.BusinessObject;
import com.softslate.commerce.businessobjects.product.Manufacturer;
import com.softslate.commerce.businessobjects.product.Product;

/**
 * Interface representing an order item within the system. An order item
 * corresponds to a single
 * {@link com.softslate.commerce.businessobjects.product.Product} that a
 * customer purchases in an order.
 * <p>
 * When the application needs to create an instance that implements
 * <code>OrderItem</code>,
 * {@link com.softslate.commerce.businessobjects.core.BusinessObjectFactory}
 * finds the name of the Java class to instantiate from the
 * "orderItemImplementer" setting in the <code>appComponents.properties</code>
 * file.
 * <p>
 * The default "orderItemImplementer" is
 * {@link com.softslate.commerce.businessobjects.order.OrderItemBean}.
 * 
 * @author David Tobey
 * 
 */
@Audited
@AuditTable("AudOrderItem")
public interface OrderItem extends BusinessObject {
	abstract int getOrderItemID();

	abstract void setOrderItemID(int orderItemID);

	abstract int getOrderDeliveryID();

	abstract void setOrderDeliveryID(int orderDeliveryID);

	abstract int getOrderID();

	abstract void setOrderID(int orderID);

	abstract Double getTotal();

	abstract void setTotal(Double total);

	abstract String getFormattedTotal();

	abstract void setFormattedTotal(String formattedTotal);

	abstract Double getWeight();

	abstract void setWeight(Double weight);
	
	abstract Double getCostTotal();

	abstract void setCostTotal(Double costTotal);

	abstract int getQuantity();

	abstract void setQuantity(int quantity);

	abstract boolean getIsActive();

	abstract void setIsActive(boolean isActive);

	abstract int getProductID();

	abstract void setProductID(int productID);

	abstract Integer getManufacturerID();

	abstract void setManufacturerID(Integer manufacturerID);

	abstract String getCode();

	abstract void setCode(String code);

	abstract String getSeoCode();

	abstract void setSeoCode(String seoCode);

	abstract String getName();

	abstract void setName(String name);

	abstract Double getUnitPrice();

	abstract void setUnitPrice(Double unitPrice);

	abstract Double getUnitCost();

	abstract void setUnitCost(Double unitCost);

	abstract Double getAltPrice();

	abstract void setAltPrice(Double altPrice);

	abstract Double getProductWeight();

	abstract void setProductWeight(Double productWeight);

	abstract String getFormattedCost();

	abstract void setFormattedCost(String formattedCost);

	abstract String getFormattedPrice();

	abstract void setFormattedPrice(String formattedPrice);

	abstract String getFormattedAltPrice();

	abstract void setFormattedAltPrice(String formattedAltPrice);

	abstract boolean getIsTaxed();

	abstract void setIsTaxed(boolean isTaxed);

	abstract String getImage();

	abstract void setImage(String image);

	abstract String getExtra1();

	abstract void setExtra1(String extra1);

	abstract String getExtra2();

	abstract void setExtra2(String extra2);

	abstract String getExtra3();

	abstract void setExtra3(String extra3);

	abstract String getExtra4();

	abstract void setExtra4(String extra4);

	abstract String getExtra5();

	abstract void setExtra5(String extra5);

	abstract String getShortDescription();

	abstract void setShortDescription(String shortDescription);

	abstract String getDecrementedSKUs();

	abstract void setDecrementedSKUs(String decrementedSKUs);

	abstract Collection getOrderItemAttributes();

	abstract void setOrderItemAttributes(Collection orderItemAttributes);

	abstract Order getOrder();

	abstract void setOrder(Order order);

	abstract OrderDelivery getOrderDelivery();

	abstract void setOrderDelivery(OrderDelivery orderDelivery);

	abstract Product getProduct();

	abstract void setProduct(Product product);

	abstract Manufacturer getManufacturer();

	abstract void setManufacturer(Manufacturer manufacturer);

	@NotAudited
	abstract Collection getOrderDiscounts();

	abstract void setOrderDiscounts(Collection orderDiscounts);

	@NotAudited
	abstract Collection getOrderShippingRules();

	abstract void setOrderShippingRules(Collection orderShippingRules);

	@NotAudited
	abstract Collection getDiscounts();

	abstract void setDiscounts(Collection discounts);

	abstract void addFormatting(Locale locale, String currencyCode);

	public double getItemDiscountTotal();

	public double getItemUnitPrice();
}
