package com.softslate.commerce.businessobjects.product;

import java.util.HashMap;
import java.util.Map;

import org.apache.commons.beanutils.BeanUtils;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import com.softslate.commerce.businessobjects.core.BaseBusinessProcessor;
import com.softslate.commerce.daos.product.DiscountRangeDAO;
import com.softslate.commerce.daos.product.DiscountRangeGatewayDAO;

/**
 * Processes business logic having to do with discount ranges. Created in the
 * Struts layer in response to requests for viewing, editing, adding, and
 * deleting discount ranges.
 * <p>
 * <code>BasicDiscountRangeProcessor</code> is the default implementation of
 * the <code>DiscountRangeProcessor</code> interface for the application.
 * 
 * @author David Tobey
 */
public class BasicDiscountRangeProcessor extends BaseBusinessProcessor
        implements DiscountRangeProcessor {

    static Log log = LogFactory.getLog(BasicDiscountRangeProcessor.class);

    public Map loadDiscountRangesAndCount(Map parameters) throws Exception {
        if (log.isInfoEnabled() && getUser().getAdministrator() != null)
            log.info(getUser().getAdministrator().getUserName()
                    + ": Loading discounts and count");
        DiscountRangeGatewayDAO gatewayDAO =  getDaoFactory()
                .createDAO(DiscountRangeGatewayDAO.class);
        return gatewayDAO.loadDiscountRangesAndCount(parameters);
    }

	public Map addDiscountRange(Map parameters) throws Exception {
        if (log.isInfoEnabled() && getUser().getAdministrator() != null) {
            String discountRangeID = (String) parameters.get("discountRangeID");
            log.info(getUser().getAdministrator().getUserName()
                    + ": Adding discount range: " + discountRangeID);
        }
		Map results = new HashMap();
        DiscountRange discountRange =  getBusinessObjectFactory()
                .createObject(DiscountRange.class);
        BeanUtils.copyProperties(discountRange, parameters);
        Discount discount = (Discount) getBusinessObjectFactory().createObject(
                "discountImplementer");
        discount.setDiscountID(discountRange.getDiscountID());
        discountRange.setDiscount(discount);
        DiscountRangeDAO dao = (DiscountRangeDAO) getDaoFactory().createDAO(
                "discountRangeDAOImplementer");
        dao.setDiscountRange(discountRange);
        dao.insertDiscountRange(true);
		results.put("newRecord", dao.getDiscountRange());
		return results;
    }

    public DiscountRange loadDiscountRangeFromID(Map parameters)
            throws Exception {
        if (log.isInfoEnabled() && getUser().getAdministrator() != null)
            log.info(getUser().getAdministrator().getUserName()
                    + ": Loading discount range from id");
        DiscountRange discountRange =  getBusinessObjectFactory()
                .createObject(DiscountRange.class);
        DiscountRangeDAO dao = (DiscountRangeDAO) getDaoFactory().createDAO(
                "discountRangeDAOImplementer");
        BeanUtils.copyProperties(discountRange, parameters);
        dao.setDiscountRange(discountRange);
        dao.loadDiscountRangeFromID();
		return dao.getDiscountRange();
    }

	public Map editDiscountRange(Map parameters) throws Exception {
        if (log.isInfoEnabled() && getUser().getAdministrator() != null) {
            String discountRangeID = (String) parameters.get("discountRangeID");
            log.info(getUser().getAdministrator().getUserName()
                    + ": Editing discount range: " + discountRangeID);
        }
		Map results = new HashMap();
		DiscountRange discountRange = loadDiscountRangeFromID(parameters);
        BeanUtils.copyProperties(discountRange, parameters);
        Discount discount = (Discount) getBusinessObjectFactory().createObject(
                "discountImplementer");
        discount.setDiscountID(discountRange.getDiscountID());
        discountRange.setDiscount(discount);
        DiscountRangeDAO dao = (DiscountRangeDAO) getDaoFactory().createDAO(
                "discountRangeDAOImplementer");
        dao.setDiscountRange(discountRange);
        dao.updateDiscountRange(true);
		results.put("updatedRecord", dao.getDiscountRange());
		return results;
    }

	public Map editDeleteDiscountRanges(Map parameters) throws Exception {
		if (log.isInfoEnabled() && getUser().getAdministrator() != null)
			log.info(getUser().getAdministrator().getUserName() + ": Editing/deleting multiple discount ranges");
		DiscountRangeGatewayDAO dao = (DiscountRangeGatewayDAO) getDaoFactory().createDAO(
				"discountRangeGatewayDAOImplementer");
		Map results = dao.editDeleteDiscountRanges(parameters, getBusinessObjectFactory());
		return results;
	}

    public void deleteDiscountRange(Map parameters) throws Exception {
        if (log.isInfoEnabled() && getUser().getAdministrator() != null) {
            String discountRangeID = (String) parameters.get("discountRangeID");
            log.info(getUser().getAdministrator().getUserName()
                    + ": Deleting discount range: " + discountRangeID);
        }
        DiscountRange discountRange =  getBusinessObjectFactory()
                .createObject(DiscountRange.class);
        BeanUtils.copyProperties(discountRange, parameters);
        DiscountRangeDAO dao = (DiscountRangeDAO) getDaoFactory().createDAO(
                "discountRangeDAOImplementer");
        dao.setDiscountRange(discountRange);
        dao.deleteDiscountRange(true);
        return;
    }
}
