package com.softslate.commerce.businessobjects.product;

import java.util.Map;

import com.softslate.commerce.businessobjects.core.BusinessProcessor;

/**
 * Interface for business logic methods having to do with category settings.
 * Created in the Struts layer in response to requests for viewing, editing,
 * adding, and deleting category settings.
 * <p>
 * When the application needs to create an instance that implements
 * <code>CategorySettingProcessor</code>,
 * {@link com.softslate.commerce.businessobjects.core.BusinessObjectFactory}
 * finds the name of the Java class to instantiate from the
 * "categorySettingProcessorImplementer" setting in the
 * <code>appComponents.properties</code> file.
 * <p>
 * The default "categorySettingProcessorImplementer" is
 * {@link com.softslate.commerce.businessobjects.product.CategorySettingProcessor}.
 * 
 * @author David Tobey
 */
public interface CategorySettingProcessor extends BusinessProcessor {

	abstract void addCategorySetting(Map parameters) throws Exception;

	abstract CategorySetting loadCategorySettingFromID(Map parameters)
			throws Exception;

	abstract void editCategorySetting(Map parameters) throws Exception;

	abstract void deleteCategorySetting(Map parameters) throws Exception;
}
