package com.softslate.commerce.businessobjects.product;

import java.io.Serializable;
import java.util.Collection;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.hibernate.envers.AuditTable;
import org.hibernate.envers.Audited;
import org.hibernate.envers.NotAudited;

import com.softslate.commerce.businessobjects.core.BaseBusinessObject;

/**
 * Object representing a manufacturer within the system.
 * <code>ManufacturerBean</code> is the default implementation of the
 * <code>Manufacturer</code> interface for the application.
 * <p>
 * The database stores manufacturers in the <code>sscManufacturer</code>
 * table, and this interface can be used to represent a single row of that
 * table. The Hibernate configuration file <code>ManufacturerBean.hbm.xml</code>
 * maps that table's columns to the properties in this object.
 * 
 * @author David Tobey
 */
@Audited
@AuditTable("AudManufacturer")
public class ManufacturerBean extends BaseBusinessObject implements
		Manufacturer, Serializable {

	private static final long serialVersionUID = -3423619179638105107L;

	static Log log = LogFactory.getLog(ManufacturerBean.class);

	private int manufacturerID = 0;

	private String code = null;

	private String name = null;

	private String websiteURL = null;

	private String description = null;

	private boolean isActive = true;

	private String header = null;

	private String footer = null;

	private String smallImage;

	private String largeImage;

	private String address1 = null;

	private String address2 = null;

	private String city = null;

	private String state = null;

	private String postalCode = null;

	private String country = null;

	private String email = null;

	private String phone = null;

	private boolean isDropShipper = false;

	private String extra1 = null;

	private String extra2 = null;

	private String extra3 = null;

	private int manufacturerOrder = 0;

	private Collection products = null;

	@NotAudited
	public Collection getProducts() {
		return products;
	}

	public void setProducts(Collection products) {
		this.products = products;
	}

	public String getCode() {
		return code;
	}

	public void setCode(String code) {
		this.code = code;
	}

	public String getDescription() {
		return description;
	}

	public void setDescription(String description) {
		this.description = description;
	}

	public String getExtra1() {
		return extra1;
	}

	public void setExtra1(String extra1) {
		this.extra1 = extra1;
	}

	public String getExtra2() {
		return extra2;
	}

	public void setExtra2(String extra2) {
		this.extra2 = extra2;
	}

	public String getExtra3() {
		return extra3;
	}

	public void setExtra3(String extra3) {
		this.extra3 = extra3;
	}

	public String getFooter() {
		return footer;
	}

	public void setFooter(String footer) {
		this.footer = footer;
	}

	public String getHeader() {
		return header;
	}

	public void setHeader(String header) {
		this.header = header;
	}

	public boolean getIsActive() {
		return isActive;
	}

	public void setIsActive(boolean isActive) {
		this.isActive = isActive;
	}

	public String getLargeImage() {
		return largeImage;
	}

	public void setLargeImage(String largeImage) {
		this.largeImage = largeImage;
	}

	public int getManufacturerID() {
		return manufacturerID;
	}

	public void setManufacturerID(int manufacturerID) {
		this.manufacturerID = manufacturerID;
	}

	public int getManufacturerOrder() {
		return manufacturerOrder;
	}

	public void setManufacturerOrder(int manufacturerOrder) {
		this.manufacturerOrder = manufacturerOrder;
	}

	public String getName() {
		return name;
	}

	public void setName(String name) {
		this.name = name;
	}

	public String getSmallImage() {
		return smallImage;
	}

	public void setSmallImage(String smallImage) {
		this.smallImage = smallImage;
	}

	public String getWebsiteURL() {
		return websiteURL;
	}

	public void setWebsiteURL(String websiteURL) {
		this.websiteURL = websiteURL;
	}

	private Collection productListCollection = null;

	public Collection getProductListCollection() {
		return (this.productListCollection);
	}

	public void setProductListCollection(Collection productListCollection) {
		this.productListCollection = productListCollection;
	}

	private int itemsPerPage = 9;

	public int getItemsPerPage() {
		return (this.itemsPerPage);
	}

	public void setItemsPerPage(int itemsPerPage) {
		this.itemsPerPage = itemsPerPage;
	}

	private int firstRow = 1;

	public int getFirstRow() {
		return (this.firstRow);
	}

	public void setFirstRow(int firstRow) {
		this.firstRow = firstRow;
	}

	private int itemCount = 0;

	public int getItemCount() {
		return (this.itemCount);
	}

	public void setItemCount(int itemCount) {
		this.itemCount = itemCount;
	}

	public String getAddress1() {
		return address1;
	}

	public void setAddress1(String address1) {
		this.address1 = address1;
	}

	public String getAddress2() {
		return address2;
	}

	public void setAddress2(String address2) {
		this.address2 = address2;
	}

	public String getCity() {
		return city;
	}

	public void setCity(String city) {
		this.city = city;
	}

	public String getState() {
		return state;
	}

	public void setState(String state) {
		this.state = state;
	}

	public String getPostalCode() {
		return postalCode;
	}

	public void setPostalCode(String postalCode) {
		this.postalCode = postalCode;
	}

	public String getCountry() {
		return country;
	}

	public void setCountry(String country) {
		this.country = country;
	}

	public String getEmail() {
		return email;
	}

	public void setEmail(String email) {
		this.email = email;
	}

	public String getPhone() {
		return phone;
	}

	public void setPhone(String phone) {
		this.phone = phone;
	}

	public boolean getIsDropShipper() {
		return isDropShipper;
	}

	public void setIsDropShipper(boolean isDropShipper) {
		this.isDropShipper = isDropShipper;
	}

}
