package com.softslate.commerce.businessobjects.shipping;

import java.util.Collection;

import org.hibernate.envers.AuditTable;
import org.hibernate.envers.Audited;

import com.softslate.commerce.businessobjects.core.BusinessObject;

/**
 * Interface representing a shipping rule within the system.
 * <p>
 * When the application needs to create an instance that implements
 * <code>sscShippingRule</code>,
 * {@link com.softslate.commerce.businessobjects.core.BusinessObjectFactory}
 * finds the name of the Java class to instantiate from the
 * "shippingRuleImplementer" setting in the <code>appComponents.properties</code>
 * file.
 * <p>
 * The default "shippingRuleImplementer" is
 * {@link com.softslate.commerce.businessobjects.shipping.ShippingRuleBean}.
 * 
 * @author Jason McWilliams
 */
@Audited
@AuditTable("AudShippingRule")
public interface ShippingRule extends BusinessObject {
	
	abstract int getShippingRuleID();
	abstract void setShippingRuleID(int shippingRuleID);

	abstract String getCode();
	abstract void setCode(String code);
	
	abstract String getName();
	abstract void setName(String name);
	
	abstract String getDescription();
	abstract void setDescription(String description);
	
	abstract boolean getIsActive();
	abstract void setIsActive(boolean isActive);
	
	abstract String getRuleType();
	abstract void setRuleType(String ruleType);
	
	abstract int getRuleOrder();
	abstract void setRuleOrder(int ruleOrder);
	
	abstract String getExtra1();
	abstract void setExtra1(String extra1);

	abstract String getExtra2();
	abstract void setExtra2(String extra2);

	abstract String getExtra3();
	abstract void setExtra3(String extra3);

	abstract Collection getSkuShippingRules();
	abstract void setSkuShippingRules(Collection skuShippingRules);

	abstract Collection getShippingRuleRanges();
	abstract void setShippingRuleRanges(Collection shippingRuleRanges);

}
