package com.softslate.commerce.customer.customer;

import java.util.Map;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.apache.struts.action.ActionForm;
import org.apache.struts.action.ActionForward;
import org.apache.struts.action.ActionMapping;
import org.apache.struts.action.ActionMessage;
import org.apache.struts.action.ActionMessages;

import com.softslate.commerce.businessobjects.customer.CustomerProcessor;
import com.softslate.commerce.customer.core.AppUtils;
import com.softslate.commerce.customer.core.BaseAction;
import com.softslate.commerce.daos.core.DAOUtils;

/**
 * Struts action class used for requests to update a customer's password and
 * user name. Used for requests to "/UpdatePassword.do".
 * <p>
 * The <code>execute</code> method of this class invokes
 * {@link com.softslate.commerce.businessobjects.customer.CustomerProcessor#updateCustomer(Map)}
 * <!-- --> to process the request and update the information in the database.
 * 
 * @author David Tobey
 * 
 */
public class UpdatePasswordAction extends BaseAction {
    static Log log = LogFactory.getLog(UpdatePasswordAction.class);

    public ActionForward execute(ActionMapping mapping, ActionForm form,
            HttpServletRequest request, HttpServletResponse response)
            throws Exception {
		if (log.isInfoEnabled())
			log.info("Starting action.");
		UpdatePasswordForm baseForm = (UpdatePasswordForm) form;
        synchronized (AppUtils.getSessionLock(request.getSession())) {

			// Preserve the original request so the customer is redirected
			// back to the main addresses page after logging in
			request.setAttribute("originalRequest", "/AccountPassword.do");
			
            // If the user is not logged in, it's an error.
            if (baseForm.getUser().getCustomer() == null) {
                baseForm.getErrors().add(ActionMessages.GLOBAL_MESSAGE,
                        new ActionMessage("errors.userNotLoggedIn"));
                saveErrors(baseForm.getRequest(), (ActionMessages) baseForm
                        .getErrors());
                if (log.isInfoEnabled() && !baseForm.getErrors().isEmpty())
                    log.info("Validation error(s): "
                            + baseForm.getErrors().toString());
                return mapping.findForward("notLoggedIn");
            }

			// Validate the current password against the logged in customer
			CustomerProcessor customerProcessor = (CustomerProcessor) baseForm.getBusinessObjectFactory()
					.createObject("customerProcessorImplementer");
			DAOUtils daoUtils = (DAOUtils) baseForm.getBusinessObjectFactory().createUtilsObject("daoUtilsImplementer");
			String currentPasswordEncrypted = daoUtils.getEncrypted(baseForm.getCurrentPassword(),
					baseForm.getSettings().getValue("customerEncryptionType"),
					customerProcessor.getAppSettings().getProperty("twoWayKeyFile"),
					baseForm.getSettings().getValue("oneWayEncryptionAlgorithm"));
			if (!baseForm.getUser().getCustomer().getPassword().equals(currentPasswordEncrypted)) {
				String message = baseForm.getMessage("customer", "customer.currentPassword", null);
				baseForm.getErrors().add(ActionMessages.GLOBAL_MESSAGE, new ActionMessage("errors.invalid", message));
				saveErrors(baseForm.getRequest(), (ActionMessages) baseForm.getErrors());
				if (log.isInfoEnabled() && !baseForm.getErrors().isEmpty())
					log.info("Validation error(s): " + baseForm.getErrors().toString());
				return mapping.findForward("failure");
			}

			Map result = customerProcessor.processChangePassword(baseForm.getCurrentPassword(),
					baseForm.getDecryptedPassword());
			if (result != null && result.get("resultCode") != null && result.get("resultCode").equals("errors.sameAsPreviousPassword")) {
					baseForm.getErrors().add(ActionMessages.GLOBAL_MESSAGE, new ActionMessage("errors.sameAsPreviousPassword",
							baseForm.getSettings().getValue("numberOfPreviousPasswordsToCheckCustomer")));
					saveErrors(baseForm.getRequest(), (ActionMessages) baseForm.getErrors());
					if (log.isInfoEnabled() && !baseForm.getErrors().isEmpty())
						log.info("Validation error(s): " + baseForm.getErrors().toString());
					return mapping.findForward("failure");
			} else if (result != null && result.get("resultCode") != null
                    && !result.get("resultCode").equals("0")) {
                String resultCode = (String) result.get("resultCode");
                baseForm.getErrors().add(ActionMessages.GLOBAL_MESSAGE,
                        new ActionMessage(resultCode));
                saveErrors(baseForm.getRequest(), (ActionMessages) baseForm
                        .getErrors());
                if (log.isInfoEnabled() && !baseForm.getErrors().isEmpty())
                    log.info("Validation error(s): "
                            + baseForm.getErrors().toString());
                return mapping.findForward("failure");
            }
            
            // Add a message to tell the user it worked
            baseForm.getMessages().add(ActionMessages.GLOBAL_MESSAGE,
                    new ActionMessage("messages.passwordUpdated"));
            saveMessages(baseForm.getRequest(), baseForm.getMessages());
            return mapping.findForward("success");
        }
    }
}
