package com.softslate.commerce.customer.customer;

import java.util.Map;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.apache.commons.beanutils.PropertyUtils;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.apache.struts.action.ActionForm;
import org.apache.struts.action.ActionForward;
import org.apache.struts.action.ActionMapping;
import org.apache.struts.action.ActionMessage;
import org.apache.struts.action.ActionMessages;

import com.softslate.commerce.businessobjects.core.Settings;
import com.softslate.commerce.businessobjects.customer.CustomerWishListProcessor;
import com.softslate.commerce.customer.core.AppUtils;
import com.softslate.commerce.customer.core.BaseAction;

/**
 * Struts action class used for requests to edit and add wish lists
 * 
 * @author Jason McWilliams
 * 
 */
public class WishListAddAction extends BaseAction {
	static Log log = LogFactory.getLog(WishListAddAction.class);

	public ActionForward execute(ActionMapping mapping, ActionForm form,
			HttpServletRequest request, HttpServletResponse response)
			throws Exception {
		if (log.isInfoEnabled())
			log.info("Starting action.");
		WishListAddEditForm baseForm = (WishListAddEditForm) form;
		synchronized (AppUtils.getSessionLock(request.getSession())) {
			// If the user is not logged in, it's an error.
			if (baseForm.getUser().getCustomer() == null) {
				baseForm.getErrors().add(ActionMessages.GLOBAL_MESSAGE,
						new ActionMessage("errors.userNotLoggedIn"));
				saveErrors(baseForm.getRequest(), (ActionMessages) baseForm
						.getErrors());
				if (log.isInfoEnabled() && !baseForm.getErrors().isEmpty())
					log.info("Validation error(s): "
							+ baseForm.getErrors().toString());
				return mapping.findForward("notLoggedIn");
			}

			// If the system is not allowing wish lists, error
			Settings settings = (Settings) servlet.getServletContext()
					.getAttribute("settings");
			String allowCustomerWishLists = (String) settings
					.getValue("allowCustomerWishLists");
			if (allowCustomerWishLists == null
					|| !allowCustomerWishLists.equals("1")) {
				baseForm.getErrors().add(ActionMessages.GLOBAL_MESSAGE,
						new ActionMessage("errors.wishListsNotAllowed"));
				saveErrors(baseForm.getRequest(), (ActionMessages) baseForm
						.getErrors());
				if (log.isInfoEnabled() && !baseForm.getErrors().isEmpty())
					log.info("Validation error(s): "
							+ baseForm.getErrors().toString());
				return mapping.findForward("failure");
			}

			int currentNumberOfLists = 0;
			if (baseForm.getUser() != null
					&& baseForm.getUser().getCustomer() != null
					&& baseForm.getUser().getCustomer().getCustomerWishLists() != null) {
				currentNumberOfLists = baseForm.getUser().getCustomer()
						.getCustomerWishLists().size();
			}

			int maxCustomerWishLists = 0;
			try {
				String maxCustomerWishListsString = (String) settings
						.getValue("maxCustomerWishLists");
				maxCustomerWishLists = Integer
						.parseInt(maxCustomerWishListsString);
			} catch (Exception e) {
				log.error(e.toString(), e);
			}

			if (maxCustomerWishLists <= currentNumberOfLists) {
				baseForm.getErrors().add(ActionMessages.GLOBAL_MESSAGE,
						new ActionMessage("errors.wishListsMax"));
				saveErrors(baseForm.getRequest(), (ActionMessages) baseForm
						.getErrors());
				if (log.isInfoEnabled() && !baseForm.getErrors().isEmpty())
					log.info("Validation error(s): "
							+ baseForm.getErrors().toString());
				return mapping.findForward("failure");
			}

			CustomerWishListProcessor customerWishListProcessor = (CustomerWishListProcessor) baseForm
					.getBusinessObjectFactory().createObject(
							"customerWishListProcessorImplementer");
			Map result = customerWishListProcessor
					.customerAddWishList(PropertyUtils.describe(baseForm));
			if (result != null && result.get("resultCode") != null
					&& !result.get("resultCode").equals("0")) {
				String resultCode = (String) result.get("resultCode");
				baseForm.getErrors().add(ActionMessages.GLOBAL_MESSAGE,
						new ActionMessage(resultCode));
				saveErrors(baseForm.getRequest(), (ActionMessages) baseForm
						.getErrors());
				if (log.isInfoEnabled() && !baseForm.getErrors().isEmpty())
					log.info("Validation error(s): "
							+ baseForm.getErrors().toString());
				return mapping.findForward("failure");
			}

			// Add a message to tell the user it worked
			baseForm.getMessages().add(ActionMessages.GLOBAL_MESSAGE,
					new ActionMessage("messages.wishListAdded"));
			saveMessages(baseForm.getRequest(), baseForm.getMessages());
			return mapping.findForward("success");
		}
	}
}
