package com.softslate.commerce.customer.order;

import java.util.Collection;
import java.util.Iterator;
import java.util.Map;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.apache.commons.beanutils.PropertyUtils;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.apache.struts.action.ActionForm;
import org.apache.struts.action.ActionForward;
import org.apache.struts.action.ActionMapping;
import org.apache.struts.action.ActionMessage;
import org.apache.struts.action.ActionMessages;

import com.softslate.commerce.businessobjects.order.CartProcessor;
import com.softslate.commerce.businessobjects.order.OrderItem;
import com.softslate.commerce.customer.core.ActionUtils;
import com.softslate.commerce.customer.core.AppUtils;
import com.softslate.commerce.customer.core.BaseForm;

/**
 * Struts action class for submissions to edit an item in the current user's
 * cart. Used for requests to "/CartItemEdit.do".
 * <p>
 * The <code>execute</code> method of this class extends
 * {@link com.softslate.commerce.customer.order.CartAddForm#validate(ActionMapping, HttpServletRequest)}
 * to process the edit item request and update the item in the database. It
 * invokes
 * {@link com.softslate.commerce.businessobjects.order.CartProcessor#processEditItem(Map)},
 * which first adds a new order item to the cart and then removes the old one.
 * 
 * @author David Tobey
 * 
 */
public class CartItemEditAction extends CartAddAction {

	static Log log = LogFactory.getLog(CartAddAction.class);

	public ActionForward execute(ActionMapping mapping, ActionForm form,
			HttpServletRequest request, HttpServletResponse response)
			throws Exception {
		if (log.isInfoEnabled())
			log.info("Starting action.");
		BaseForm baseForm = (BaseForm) form;

		synchronized (AppUtils.getSessionLock(request.getSession())) {

			// Check that the user's session has not expired.
			if (baseForm.getUser() == null
					|| baseForm.getUser().getOrder() == null) {
				baseForm.getErrors().add(ActionMessages.GLOBAL_MESSAGE,
						new ActionMessage("errors.sessionExpired"));
			} else if (baseForm.formUtils()
					.isInteger(
							(String) PropertyUtils.getProperty(baseForm,
									"orderItemID"))) {

				// Check that the orderID is in the user's cart.
				int orderItemID = Integer.parseInt((String) PropertyUtils
						.getProperty(baseForm, "orderItemID"));

				// Try to find the OrderItem
				if (baseForm.getUser().getOrder().firstDelivery() != null
						&& baseForm.getUser().getOrder().firstDelivery()
								.getOrderItemCollection() != null) {
					Collection ois = baseForm.getUser().getOrder()
							.firstDelivery().getOrderItemCollection();
					Iterator i = ois.iterator();
					while (i.hasNext()) {
						OrderItem oi = (OrderItem) i.next();
						if (oi.getOrderItemID() == orderItemID) {
							PropertyUtils
									.setProperty(baseForm, "orderItem", oi);
							break;
						}
					}
				}

				// Error if the OrderItem can't be found
				if (PropertyUtils.getProperty(baseForm, "orderItem") == null) {
					baseForm.getErrors().add(ActionMessages.GLOBAL_MESSAGE,
							new ActionMessage("errors.orderItemNotFound"));
					saveErrors(baseForm.getRequest(), (ActionMessages) baseForm
							.getErrors());
					return baseForm.getMapping().findForward("failure");
				}
			}

			// Process the editing of the item
			baseForm.getRequest().getSession().setAttribute(
					"shippingAndTaxesProcessed", "false");
			CartProcessor cartProcessor = (CartProcessor) baseForm
					.getBusinessObjectFactory().createObject(
							"cartProcessorImplementer");
			Map results = cartProcessor.processEditItem(PropertyUtils
					.describe(baseForm));
			ActionUtils actionUtils = (ActionUtils) baseForm
					.getBusinessObjectFactory().createUtilsObject(
							"actionUtilsImplementer");
			String forward = actionUtils.processCartAddResults(results,
					baseForm);
			actionUtils.loadEstimatedShippingOptions(baseForm);
			return mapping.findForward(forward);
		}
	}

}
