package com.softslate.commerce.customer.order;

import javax.servlet.http.HttpServletRequest;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.apache.struts.action.ActionErrors;
import org.apache.struts.action.ActionMapping;
import org.apache.struts.action.ActionMessage;
import org.apache.struts.action.ActionMessages;

import com.softslate.commerce.customer.core.AppUtils;
import com.softslate.commerce.customer.core.BaseForm;

/**
 * Struts form class representing the delivery information prompted for on the
 * cart page to calculated estimated shipping. It s used for submissions to
 * "/EstimatedShipping.do".
 * <p>
 * The <code>validate</code> method of this class first checks to see if the
 * user submitted values for the delivery postal code, state and country, which
 * are required to calculate shipping.
 * 
 * @author David Tobey
 * 
 */
public class EstimatedShippingForm extends BaseForm {
	private static final long serialVersionUID = 1L;

	static Log log = LogFactory.getLog(EstimatedShippingForm.class);

	private String deliveryCity = null;

	private String deliveryState = null;

	private String deliveryPostalCode = null;

	private String deliveryCountry = null;

	public String getDeliveryCity() {
		return deliveryCity;
	}

	public void setDeliveryCity(String deliveryCity) {
		this.deliveryCity = deliveryCity;
	}

	public String getDeliveryState() {
		return deliveryState;
	}

	public void setDeliveryState(String deliveryState) {
		this.deliveryState = deliveryState;
	}

	public String getDeliveryPostalCode() {
		return deliveryPostalCode;
	}

	public void setDeliveryPostalCode(String deliveryPostalCode) {
		this.deliveryPostalCode = deliveryPostalCode;
	}

	public String getDeliveryCountry() {
		return deliveryCountry;
	}

	public void setDeliveryCountry(String deliveryCountry) {
		this.deliveryCountry = deliveryCountry;
	}

	public ActionErrors validate(ActionMapping mapping,
			HttpServletRequest request) {
		if (log.isInfoEnabled())
			log.info("Starting validation.");
		initializeProperties(mapping, request);

		synchronized (AppUtils.getSessionLock(request.getSession())) {

			// Set a flag in the session if the fields are valid
			getRequest().getSession().setAttribute(
					"estimatedShippingFormValidated", null);

			// Specifically because of UPS's requirements, we need a city if the
			// country is not the US.
			if (getDeliveryCountry() != null
					&& !getDeliveryCountry().equals("US")
					&& (getDeliveryCity() == null || getDeliveryCity().equals(
							""))) {
				String message = getMessage("order", "orderForm.deliveryCity",
						null);
				getErrors().add(ActionMessages.GLOBAL_MESSAGE,
						new ActionMessage("errors.required", message));
			}

			// A ZIP code is required if the country is the US.
			if (getDeliveryCountry() != null
					&& getDeliveryCountry().equals("US")
					&& (getDeliveryPostalCode() == null || getDeliveryPostalCode()
							.equals(""))) {
				String message = getMessage("order",
						"orderForm.deliveryPostalCode", null);
				getErrors().add(ActionMessages.GLOBAL_MESSAGE,
						new ActionMessage("errors.required", message));
			}

			if (getDeliveryCountry() == null || getDeliveryCountry().equals("")) {
				String message = getMessage("order",
						"orderForm.deliveryCountry", null);
				getErrors().add(ActionMessages.GLOBAL_MESSAGE,
						new ActionMessage("errors.required", message));
			}

			// Set a flag in the session if the fields are valid. Tells
			// ActionUtils.loadEstimatedShippingOptions to load shipping
			// options.
			if (getErrors().isEmpty()) {
				getRequest().getSession().setAttribute(
						"estimatedShippingFormValidated", new Boolean(true));
			}
		}

		if (log.isDebugEnabled())
			log.debug("Finished validation.");
		if (log.isInfoEnabled() && !getErrors().isEmpty())
			log.info("Validation error(s): " + getErrors().toString());
		return getErrors();
	}
}
