package com.softslate.commerce.customer.product;

import javax.servlet.http.HttpServletRequest;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.apache.struts.action.ActionErrors;
import org.apache.struts.action.ActionMapping;
import org.apache.struts.action.ActionMessage;
import org.apache.struts.action.ActionMessages;

import com.softslate.commerce.customer.core.BaseDynaForm;

/**
 * Struts form class that correspondes to the manufacturer page. Used for
 * requests to "/Manufacturer.do".
 * <p>
 * The <code>validate</code> method of this class simply makes sure that a
 * manufacturer <code>code</code> of non-zero length was submitted.
 * 
 * @author David Tobey
 * 
 * @see com.softslate.commerce.customer.product.ManufacturerAction
 */
public class ManufacturerForm extends BaseDynaForm {

	private static final long serialVersionUID = 8389810488305656220L;

	static Log log = LogFactory.getLog(ManufacturerForm.class);

	public ActionErrors validate(ActionMapping mapping,
			HttpServletRequest request) {
		if (log.isInfoEnabled())
			log.info("Starting validation.");
		initializeProperties(mapping, request);
		if (!getFormUtils().stringExists(this, "code")) {
			String message = getMessage("product", "product.code", null);
			getErrors().add(ActionMessages.GLOBAL_MESSAGE,
					new ActionMessage("errors.required", message));
		}

		// Remove any null UTF characters to avoid an Exception from Postgres
		set("code", ((String) get("code")).replaceAll("\u0000", ""));
		if (log.isInfoEnabled() && !getErrors().isEmpty())
			log.info("Validation error(s): " + getErrors().toString());
		return getErrors();
	}

}
