package com.softslate.commerce.customer.product;

import javax.servlet.RequestDispatcher;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.apache.struts.action.ActionForm;
import org.apache.struts.action.ActionForward;
import org.apache.struts.action.ActionMapping;
import org.apache.struts.action.ActionMessage;
import org.apache.struts.action.ActionMessages;

import com.softslate.commerce.businessobjects.product.Product;
import com.softslate.commerce.businessobjects.product.ProductProcessor;
import com.softslate.commerce.customer.core.AppUtils;
import com.softslate.commerce.customer.core.BaseAction;

/**
 * Class to load Product Review Form for Review Submission
 * 
 * @author Tony Zakula
 */

public class ProductReviewFormAction extends BaseAction {

	static Log log = LogFactory.getLog(ProductReviewFormAction.class);

	public ActionForward execute(ActionMapping mapping, ActionForm form, HttpServletRequest request,
			HttpServletResponse response) throws Exception {
		if (log.isInfoEnabled())
			log.info("Starting Create Product Review Action.");

		ProductReviewForm baseForm = (ProductReviewForm) form;
		baseForm.initializeProperties(mapping, request);

		synchronized (AppUtils.getSessionLock(request.getSession())) {

			// Preserve the original request so the customer is redirected
			// back here after logging in
			request.setAttribute("originalRequest", request.getRequestURI() + "?code=" + baseForm.getCode());

			// If the user is not logged in, it's an error.
			if (baseForm.getUser().getCustomer() == null) {
				String url = "/AccountLogin.do";

				baseForm.getErrors().add(ActionMessages.GLOBAL_MESSAGE, new ActionMessage("errors.reviewLogin"));
				saveErrors(baseForm.getRequest(), (ActionMessages) baseForm.getErrors());

				RequestDispatcher rd = request.getRequestDispatcher(url);
				rd.forward(request, response);
			}

			// User is logged in so continue processing
			if (baseForm.getCode() == null || baseForm.getCode().length() == 0) {
				baseForm.getErrors().add(ActionMessages.GLOBAL_MESSAGE, new ActionMessage("errors.productNotFound"));
				saveErrors(baseForm.getRequest(), (ActionMessages) baseForm.getErrors());
				return mapping.findForward("failure");
			}

			// Get the product data from the database
			ProductProcessor productProcessor = (ProductProcessor) baseForm.getBusinessObjectFactory().createObject(
					"productProcessorImplementer");
			Product product =  baseForm.getBusinessObjectFactory().createObject(Product.class);
			product.setCode(baseForm.getCode());
			product = productProcessor.getProductFromCode(product);

			// If that fails, it's an error
			if (product.getProductID() == 0) {
				baseForm.getErrors().add(ActionMessages.GLOBAL_MESSAGE,
						new ActionMessage("errors.productNotFound", product.getCode()));
				saveErrors(baseForm.getRequest(), (ActionMessages) baseForm.getErrors());
				return mapping.findForward("failure");
			}

			// Now populate the form data with their user data if it is there
			if (baseForm.getUser() != null && baseForm.getUser().getCustomer() != null) {
				baseForm.setFirstName(baseForm.getUser().getCustomer().getFirstName());
				baseForm.setLastName(baseForm.getUser().getCustomer().getLastName());
				baseForm.setState(baseForm.getUser().getCustomer().getState());
				baseForm.setCity(baseForm.getUser().getCustomer().getCity());
				baseForm.setCountry(baseForm.getUser().getCustomer().getCountry());
				baseForm.setNickname(baseForm.getUser().getCustomer().getFirstName());
				if (baseForm.getFirstName() != null && !baseForm.getFirstName().equals("")
						&& baseForm.getLastName() != null && !baseForm.getLastName().equals("")) {
					baseForm.setNickname(baseForm.getFirstName() + " " + baseForm.getLastName().substring(0, 1) + ".");
				}
			}

			request.setAttribute("product", product);
			request.setAttribute("productReviewForm", baseForm);

			return mapping.findForward("success");

		} // End of sync
	} // End of execute statement

}
