package com.softslate.commerce.daos.administrator;

import org.apache.commons.beanutils.PropertyUtils;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.hibernate.HibernateException;
import org.hibernate.Query;
import org.hibernate.Session;

import com.softslate.commerce.businessobjects.administrator.Role;
import com.softslate.commerce.daos.core.BaseDAO;
import com.softslate.commerce.daos.core.DataAccessException;

/**
 * Generic DAO to manage the <code>sscRole</code> table of the database.
 * 
 * @author David Tobey
 * 
 */
public class RoleDAOHibernate extends BaseDAO implements RoleDAO {

    static Log log = LogFactory.getLog(RoleDAOHibernate.class);

    private Role role = null;

    public Role getRole() {
        return (this.role);
    }

    public void setRole(Role role) {
        this.role = role;
    }

    public void loadRoleFromID() throws Exception {
        Session session = getDaoFactory().startSession();
        try {
            Role record = (Role) session.get(Role.class, new Integer(getRole()
                    .getRoleID()));
            PropertyUtils.copyProperties(getRole(), record);
			setRole(record);
        } catch (HibernateException he) {
            if (log.isErrorEnabled())
                log.error("Hibernate error: " + he.toString());
            throw new DataAccessException(he);
        }
        return;
    }

    public void loadRoleFromCode() throws DataAccessException {
        Session session = getDaoFactory().startSession();
        try {
            Query q = getNamedQuery(session, "role.selectFromCode");
            q.setString("code", getRole().getCode());
            setRole((Role) q.uniqueResult());
        } catch (HibernateException he) {
            if (log.isErrorEnabled())
                log.error("Hibernate error: " + he.toString());
            throw new DataAccessException(he);
        }
        return;
    }

    public void insertRole() throws DataAccessException {
        insertRole(false);
    }

    public void insertRole(boolean commit) throws DataAccessException {
        Session session = getDaoFactory().startSession();
        try {
            session.save(getRole());
            if (commit) {
                getDaoFactory().commitTransaction();
            }
        } catch (HibernateException he) {
            if (log.isErrorEnabled())
                log.error("Hibernate error: " + he.toString());
            throw new DataAccessException(he);
        }
        return;
    }

    public void updateRole() throws DataAccessException {
        updateRole(false);
    }

    public void updateRole(boolean commit) throws DataAccessException {
        Session session = getDaoFactory().startSession();
        try {
            session.update(getRole());
            if (commit) {
                getDaoFactory().commitTransaction();
            }
        } catch (HibernateException he) {
            if (log.isErrorEnabled())
                log.error("Hibernate error: " + he.toString());
            throw new DataAccessException(he);
        }
        return;
    }

    public void deleteRole() throws DataAccessException {
        deleteRole(false);
    }

    public void deleteRole(boolean commit) throws DataAccessException {
        Session session = getDaoFactory().startSession();
        try {
            setRole((Role) session.load(Role.class, new Integer(getRole()
                    .getRoleID())));
            session.delete(getRole());
            if (commit) {
                getDaoFactory().commitTransaction();
            }
        } catch (HibernateException he) {
            if (log.isErrorEnabled())
                log.error("Hibernate error: " + he.toString());
            throw new DataAccessException(he);
        }
        return;
    }

}
