package com.softslate.commerce.daos.product;

import org.apache.commons.beanutils.PropertyUtils;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.hibernate.HibernateException;
import org.hibernate.Session;

import com.softslate.commerce.businessobjects.product.CategorySetting;
import com.softslate.commerce.daos.core.BaseDAO;
import com.softslate.commerce.daos.core.DAOInterface;
import com.softslate.commerce.daos.core.DataAccessException;


/**
 * Generic DAO to manage the <code>sscCategorySetting</code> table of the
 * database.
 * 
 * @author David Tobey
 */
public class CategorySettingDAOHibernate extends BaseDAO implements
        DAOInterface, CategorySettingDAO {

    static Log log = LogFactory.getLog(CategorySettingDAOHibernate.class);

    private CategorySetting categorySetting = null;

    public CategorySetting getCategorySetting() {
        return (this.categorySetting);
    }

    public void setCategorySetting(CategorySetting categorySetting) {
        this.categorySetting = categorySetting;
    }

    public void loadCategorySettingFromID() throws Exception {
        Session session = getDaoFactory().startSession();
        try {
            CategorySetting item = (CategorySetting) session.get(
                    CategorySetting.class, new Integer(getCategorySetting()
                            .getCategorySettingID()));
            PropertyUtils.copyProperties(getCategorySetting(), item);
            setCategorySetting(item);
        } catch (HibernateException he) {
            if (log.isErrorEnabled())
                log.error("Hibernate error: " + he.toString());
            throw new DataAccessException(he);
        }
        return;
    }
    public void insertCategorySetting() throws DataAccessException {
        insertCategorySetting(false);
    }

    public void insertCategorySetting(boolean commit)
            throws DataAccessException {
        Session session = getDaoFactory().startSession();
        try {
            session.save(getCategorySetting());
            if (commit) {
                getDaoFactory().commitTransaction();
            }
            evictCache();
        } catch (HibernateException he) {
            if (log.isErrorEnabled())
                log.error("Hibernate error: " + he.toString());
            throw new DataAccessException(he);
        }
        return;
    }

    public void updateCategorySetting() throws DataAccessException {
        updateCategorySetting(false);
    }

    public void updateCategorySetting(boolean commit)
            throws DataAccessException {
        Session session = getDaoFactory().startSession();
        try {
            session.update(getCategorySetting());
            if (commit) {
                getDaoFactory().commitTransaction();
            }
        } catch (HibernateException he) {
            if (log.isErrorEnabled())
                log.error("Hibernate error: " + he.toString());
            throw new DataAccessException(he);
        }
        return;
    }

    public void deleteCategorySetting() throws DataAccessException {
        deleteCategorySetting(false);
    }

    public void deleteCategorySetting(boolean commit)
            throws DataAccessException {
        Session session = getDaoFactory().startSession();
        try {
            setCategorySetting((CategorySetting) session.load(
                    CategorySetting.class, new Integer(getCategorySetting()
                            .getCategorySettingID())));
            session.delete(getCategorySetting());
            if (commit) {
                getDaoFactory().commitTransaction();
            }
            evictCache();
        } catch (HibernateException he) {
            if (log.isErrorEnabled())
                log.error("Hibernate error: " + he.toString());
            throw new DataAccessException(he);
        }
        return;
    }
}
