package com.softslate.commerce.daos.product;

import java.util.Collection;
import java.util.LinkedHashSet;
import java.util.Map;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.hibernate.HibernateException;
import org.hibernate.Query;
import org.hibernate.Session;

import com.softslate.commerce.businessobjects.core.BusinessObjectFactory;
import com.softslate.commerce.daos.core.BaseAdminGatewayDAOHibernate;
import com.softslate.commerce.daos.core.DataAccessException;

/**
 * Generic DAO that runs miscellaneous queries related to options.
 * 
 * @author David Tobey
 * 
 */
public class OptionGatewayDAOHibernate extends BaseAdminGatewayDAOHibernate implements OptionGatewayDAO {

	static Log log = LogFactory.getLog(OptionGatewayDAOHibernate.class);

	public Map loadOptionsAndCount(Map parameters) throws Exception {
		return loadItemsAndCount(parameters);
	}

	public Map editDeleteOptions(Map parameters, BusinessObjectFactory bof) throws Exception {
		Map result = editDeleteItems(parameters, bof);
		evictCache();
		return result;
	}

	public Collection loadAllOptions() throws DataAccessException {
		Collection options = null;
		Session session = getDaoFactory().startSession();
		try {
			Query q = getNamedQuery(session, "option.selectAll");
			options = new LinkedHashSet(q.list());
		} catch (HibernateException he) {
			if (log.isErrorEnabled())
				log.error("Hibernate error: " + he.toString());
			throw new DataAccessException(he);
		}
		return options;
	}

	public int loadMaxOptionID() throws Exception {
		Session session = getDaoFactory().startSession();
		try {
			Query q = getNamedQuery(session, "product.maxOptionID");
			Integer maxNum = (Integer) q.uniqueResult();
			if (maxNum == null) {
				return 0;
			}
			return maxNum.intValue();
		} catch (HibernateException he) {
			if (log.isErrorEnabled())
				log.error("Hibernate error: " + he.toString());
			throw new DataAccessException(he);
		}
	}

	public Collection loadAllOptionsInRange(int firstID, int lastID) throws Exception {
		if (log.isInfoEnabled())
			log.info("Loading options within an id range: " + firstID + "-" + lastID);
		Collection items = null;
		Session session = getDaoFactory().startSession();
		try {
			Query q = getNamedQuery(session, "option.selectAllInRange");
			q.setInteger("firstID", firstID);
			q.setInteger("lastID", lastID);
			items = new LinkedHashSet(q.list());
		} catch (HibernateException he) {
			if (log.isErrorEnabled())
				log.error("Hibernate error: " + he.toString());
			throw new DataAccessException(he);
		}
		return items;
	}
}
